/*
 * Copyright 2021 Arne Vajhøj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.jnosql.rocksdb;

import java.io.IOException;
import java.time.Duration;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

import org.eclipse.jnosql.communication.Value;
import org.eclipse.jnosql.communication.keyvalue.BucketManager;
import org.eclipse.jnosql.communication.keyvalue.KeyValueEntity;
import org.rocksdb.RocksDB;
import org.rocksdb.RocksDBException;

import dk.vajhoej.jnosql.Deserialize;
import dk.vajhoej.jnosql.Serialize;

class RocksDBBucketManager implements BucketManager {
	private RocksDB db;
	private boolean prefix;
	private String bucket;
	private Serialize to;
	private Deserialize from;
	private byte[] keyWrap(Object key) {
		String keystr;
		if(key instanceof byte[]) {
			keystr = new String((byte[])key);
		} else if(key instanceof char[]) {
			keystr = new String((char[])key);
		} else {
			keystr = key.toString();
		}
		if(prefix) {
			keystr = bucket + ":" + keystr;
		}
		return keystr.getBytes();
	}
	public RocksDBBucketManager(String dbpath, boolean prefix, String bucket, Serialize to, Deserialize from) {
        try {
			RocksDB.loadLibrary();
			db = RocksDB.open(dbpath);
		} catch (RocksDBException e) {
			throw new RuntimeException(e);
		}
        this.prefix = prefix;
        this.bucket = bucket;
		this.to = to;
		this.from = from;
	}
	@Override
	public <K, V> void put(K key, V val) {
		try {
			db.put(keyWrap(key), to.convert(val));
		} catch (IOException e) {
			throw new RuntimeException(e);
		} catch (RocksDBException e) {
			throw new RuntimeException(e);
		}
	}
	@Override
	public void put(KeyValueEntity kve) {
		put(kve.key(), kve.value());
	}
	@Override
	public void put(Iterable<KeyValueEntity> kves) {
		for(KeyValueEntity kve : kves) {
			put(kve);
		}
	}
	@Override
	public void put(KeyValueEntity kve, Duration exp) {
		throw new UnsupportedOperationException("RocksDB does not support expiration");
	}
	@Override
	public void put(Iterable<KeyValueEntity> kves, Duration exp) {
		throw new UnsupportedOperationException("RocksDB does not support expiration");
	}
	@Override
	public <K> Optional<Value> get(K key) {
		try {
			byte[] rawvalue = db.get(keyWrap(key));
	        if(rawvalue != null) {
				return Optional.of(Value.of(from.convert(rawvalue)));
	        } else {
	        	return Optional.empty();
	        }
		} catch (RocksDBException e) {
			throw new RuntimeException(e);
		} catch (ClassNotFoundException e) {
			throw new RuntimeException(e);
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
	}
	@Override
	public <K> Iterable<Value> get(Iterable<K> keys) {
		List<Value> res = new ArrayList<Value>();
		for(K key : keys) {
			Optional<Value> temp = get(key);
			if(temp.isPresent()) {
				res.add(temp.get());
			}
		}
		return res;
	}
	@Override
	public <K> void delete(K key) {
		try {
			db.delete(keyWrap(key));
		} catch (RocksDBException e) {
			throw new RuntimeException(e);
		}
	}
	@Override
	public <K> void delete(Iterable<K> keys) {
		for(K key : keys) {
			delete(key);
		}
	}
	@Override
	public String name() {
		return "RocksDB";
	}
	@Override
	public void close() {
        db.close();
	}
}
